define( [

	"suds/events/Dispatcher"

], function( Dispatcher ) {

	describe( "events/Dispatcher", function() {

		var dispatcher, listener;


		beforeEach( function() {
			dispatcher = new Dispatcher();
			listener = sinon.spy();
		} );

		afterEach( function() {
			dispatcher.removeAllListeners();
		} );


		it ( "should call listener", function() {

			dispatcher.addListener( "name", listener );
			dispatcher.dispatch( "name" );

			expect( listener.calledOnce ).toBe( true );

		} );

		it ( "should call listener with parameter", function() {

			var event = { first: "string", second: 3 };

			dispatcher.addListener( "name", listener );
			dispatcher.dispatch( "name", event );

			expect( listener.calledOnce ).toBe( true );
			expect( listener.args[ 0 ][ 0 ] ).toBe( event );

		} );

		it ( "should not call listener after stop listening", function() {

			dispatcher.addListener( "name", listener );
			dispatcher.dispatch( "name" );
			dispatcher.removeListener( "name", listener );
			dispatcher.dispatch( "name" );

			expect( listener.calledOnce ).toBe( true );

		} );

		it ( "should test against certain listeners", function() {

			dispatcher.addListener( "name", listener );

			expect( dispatcher.hasListenerFor( "name" ) ).toBe( true );
			expect( dispatcher.hasListenerFor( "car" ) ).toBe( false );

		} );

		it ( "should remove all listeners", function() {

			dispatcher.addListener( "name", listener );
			dispatcher.addListener( "car", listener );
			dispatcher.addListener( "mathias", function() {} );

			dispatcher.removeAllListeners();

			expect( dispatcher.hasListenerFor( "name" ) ).toBe( false );
			expect( dispatcher.hasListeners() ).toBe( false );

		} );

	} );

} );