define( [

	"suds/math/Point"

], function( Point ) {

	describe( "math/Point", function() {

		var a, b;

		beforeEach( function() {
			a = new Point( 3, 6 );
			b = new Point( 3, 4 );
		} );

		it ( "instantiate with and without values", function() {

			var a = new Point( 3, 6 );
			var b = new Point();

			expect( a.x ).toBe( 3 );
			expect( a.y ).toBe( 6 );

			expect( b.x ).toBe( 0 );
			expect( b.y ).toBe( 0 );

		} );

		it ( "Point::add()", function() {

			a.add( b );

			expect( a.x ).toBe( 6 );
			expect( a.y ).toBe( 10 );

			expect( b.x ).toBe( 3 );
			expect( b.y ).toBe( 4 );

		} );

		it ( "Point::subtract()", function() {

			a.subtract( b );

			expect( a.x ).toBe( 0 );
			expect( a.y ).toBe( 2 );

			expect( b.x ).toBe( 3 );
			expect( b.y ).toBe( 4 );

		} );

		it ( "Point::multiply()", function() {

			a.multiply( 2 );

			expect( a.x ).toBe( 6 );
			expect( a.y ).toBe( 12 );

		} );

		it ( "Point::divide()", function() {

			a.divide( 3 );

			expect( a.x ).toBe( 1 );
			expect( a.y ).toBe( 2 );

		} );

		it ( "Point::clone()", function() {

			var c = a.clone();

			expect( c.x ).toBe( 3 );
			expect( c.y ).toBe( 6 );
		} );

		it ( "Point::length()", function() {

			var length = ( new Point(3, 0) ).length();

			expect( length ).toBe( 3 );
		} );

		it ( "Point::normalize()", function() {

			a.normalize();

			expect( a.length() ).toBeCloseTo( 1, 0.01 );
		} );

		it ( "Point::set()", function() {

			a.set( new Point( 15, 16 ) );

			expect( a.x ).toBe( 15 );
			expect( a.y ).toBe( 16 );
		} );

		it ( "Point.add()", function() {

			var c = Point.add( a, b );

			expect( c.x ).toBe( 6 );
			expect( c.y ).toBe( 10 );

		} );

		it ( "Point.subtract()", function() {

			var c = Point.subtract( a, b );

			expect( c.x ).toBe( 0 );
			expect( c.y ).toBe( 2 );

		} );

		it ( "Point.distance() should return the distance in between two Points as number", function() {

			var a = new Point( 4, 3 );
			var b = new Point( 8, 3 );

			var distance = Point.distance( a, b );

			expect( distance ).toBe( 4 );

		} );

		it ( "Point.between() should return the distance in between two Points as Point", function() {

			var c = Point.between( a, b );

			expect( c.x ).toBe( 0 );
			expect( c.y ).toBe( -2 );

		} );

	} );

} );