define( [

	"suds/models/Model"

], function( Model ) {

	describe( "models/Model", function() {

		var model;

		beforeEach( function() {
			model = new Model( {
				name: "name",
				age: 20,
				isYoung: true,
				values: { first: true }
			} );
		} );

		it ( "should initialize with values", function() {

			expect( model.get( "name" ) ).toBe( "name" );
			expect( model.get( "age" ) ).toBe( 20 );
			expect( model.get( "isYoung") ).toBeTruthy();
			expect( model.get( "values" ) ).toEqual( { first: true } );

		} );

		it ( "Model::set()", function() {

			var listenerA = sinon.spy();
			var listenerB = sinon.spy();

			model.set( "name", "another" );

			expect( model.get( "name" ) ).toBe( "another" );

			model.change( "name", listenerA );
			model.change( "age", listenerB );

			model.set( {
				age: 40,
				swirly: "swirly"
			} );

			expect( model.get( "age" ) ).toBe( 40 );
			expect( model.get( "swirly" ) ).toBe( "swirly" );

			expect( listenerA.called ).toBeFalsy();
			expect( listenerB.calledOnce ).toBeTruthy();

		} );

		it ( "Model::get()", function() {

			var listener = sinon.spy();
			var value = model.get( "name", listener );

			expect( value ).toBe( "name" );
			expect( listener.calledOnce ).toBeTruthy();
			expect( listener.args[ 0 ][ 0 ] ).toBe( "name" );

			model.set( "name", "another" );

			expect( listener.calledTwice ).toBeTruthy();
			expect( listener.args[ 1 ][ 0 ] ).toBe( "another" );

		} );

		it ( "Model::change()", function() {

			var listener = sinon.spy();

			model.change( "name", listener );

			expect( listener.called ).toBeFalsy();

			model.set( "name", "honolulu" );

			expect( listener.calledOnce ).toBeTruthy();

		} );

		it ( "should only call back when value changed", function() {

			var listener = sinon.spy();
			model.change( "name", listener );

			model.set( "name", "another" );
			expect( listener.calledOnce ).toBeTruthy();

			model.set( "name", "another" );
			expect( listener.calledOnce ).toBeTruthy();

			model.set( {
				name: "hello",
				age: 90
			} );

			expect( listener.calledTwice ).toBeTruthy();

			model.set( {
				name: "hello",
				age: 90
			} );

			expect( listener.calledTwice ).toBeTruthy();

			model.set( {
				surname: "hello",
				age: 90
			} );

			expect( listener.calledTwice ).toBeTruthy();


		} );

	} );

} );